@@ -8,8 +8,11 @@ if TYPE_CHECKING:
 import monaco.mc_var
 
 import numpy as np
-from scipy.stats import rv_continuous, rv_discrete, chi2, mode
 import matplotlib.pyplot as plt
+from scipy.interpolate import griddata
+from scipy.stats import rv_continuous, rv_discrete, chi2, mode
+from copy import copy, deepcopy
+from typing import Optional, Iterable
 from matplotlib.figure import Figure
 from matplotlib.axes import Axes
 from matplotlib.patches import Ellipse
@@ -18,8 +21,6 @@ from monaco.helper_functions import get_list, slice_by_index, length, empty_list
 from monaco.gaussian_statistics import conf_ellipsoid_sig2pct
 from monaco.integration_statistics import integration_error
 from monaco.mc_enums import SampleMethod, PlotOrientation, InVarSpace, Sensitivities
-from copy import copy
-from typing import Optional, Iterable
 
 
 # If cases or highlight_cases are None, will plot all. Set to [] to plot none.
@@ -73,10 +74,11 @@ def plot(varx   : InVar | OutVar,
         fig is the figure handle for the plot.
         ax is the axes handle for the plot.
     """
-    # Split larger vars
+    # Split single vars that are 2D or 3D vectors
     if vary is None and varz is None:
         if varx.maxdim not in (0, 1, 2):
-            raise ValueError(f'Invalid variable dimension: {varx.name} ({varx.maxdim})')
+            raise ValueError(f'Invalid variable dimension: {varx.name}: {varx.maxdim}')
+
         elif varx.maxdim == 2 and isinstance(varx, monaco.mc_var.OutVar):
             varx_split = varx.split()  # split only defined for OutVar
             origname = varx.name
@@ -84,19 +86,30 @@ def plot(varx   : InVar | OutVar,
             vary = varx_split[origname + ' [1]']
             if len(varx_split) == 3:
                 varz = varx_split[origname + ' [2]']
+            elif len(varx_split) > 3:
+                raise ValueError('Can only split a single variable into 3 vectors:' +
+                                 f'{origname}: {len(varx_split)}')
 
+    # Split one of two vars that is a 2D vector
     elif vary is not None and varz is None:
-        if varx.maxdim == 1 and vary.maxdim == 0 and isinstance(varx, monaco.mc_var.OutVar):
+        if varx.maxdim == 2 and vary.maxdim == 0 and isinstance(varx, monaco.mc_var.OutVar):
             varx_split = varx.split()  # split only defined for OutVar
             origname = varx.name
             varz = vary
             varx = varx_split[origname + ' [0]']
             vary = varx_split[origname + ' [1]']
-        elif varx.maxdim == 0 and vary.maxdim == 1 and isinstance(vary, monaco.mc_var.OutVar):
+            if len(varx_split) > 2:
+                raise ValueError('Can only split one of two variables into 2 vectors:' +
+                                 f'{origname}: {len(varx_split)}')
+
+        elif varx.maxdim == 0 and vary.maxdim == 2 and isinstance(vary, monaco.mc_var.OutVar):
             vary_split = vary.split()  # split only defined for OutVar
             origname = vary.name
             vary = vary_split[origname + ' [0]']
             varz = vary_split[origname + ' [1]']
+            if len(vary_split) > 2:
+                raise ValueError('Can only split one of two variables into 2 vectors:' +
+                                 f'{origname}: {len(vary_split)}')
 
     # Single Variable Plots
     if vary is None and varz is None:
@@ -105,11 +118,9 @@ def plot(varx   : InVar | OutVar,
                                 rug_plot=rug_plot, invar_space=invar_space, ax=ax, title=title,
                                 plotkwargs=plotkwargs)
         else:
-            vary = copy(varx)
-            steps = np.arange(max(len(num) for num in vary.nums))
-            vals = [steps for _ in range(varx.ncases)]
-            varsteps = monaco.mc_var.OutVar(name='Simulation Steps', vals=vals)
-            fig, ax = plot_2d_line(varx=varsteps, vary=vary,
+            var1 = copy(varx)
+            var0 = get_var_steps(varx)
+            fig, ax = plot_2d_line(varx=var0, vary=var1,
                                    highlight_cases=highlight_cases,
                                    invar_space=invar_space,
                                    ax=ax, title=title, plotkwargs=plotkwargs)
@@ -128,10 +139,24 @@ def plot(varx   : InVar | OutVar,
                                    cases=cases, highlight_cases=highlight_cases,
                                    invar_space=invar_space,
                                    ax=ax, title=title, plotkwargs=plotkwargs)
+
+        elif varx.maxdim in (0, 1) and vary.maxdim in (0, 1):
+            var2 = copy(vary)
+            if varx.maxdim == 1:
+                var1 = varx
+                var0 = get_var_steps(varx)
+            elif vary.maxdim == 1:
+                var1 = get_var_steps(vary)
+                var0 = varx
+            fig, ax = plot_2p5d_line(varx=var0, vary=var1, varz=var2,
+                                     cases=cases, highlight_cases=highlight_cases,
+                                     invar_space=invar_space,
+                                     ax=ax, title=title, plotkwargs=plotkwargs)
+
         else:
-            raise ValueError( 'Variables have inconsistent dimensions: ' +
-                             f'{varx.name}:{varx.maxdim}, ' +
-                             f'{vary.name}:{vary.maxdim}')
+            raise ValueError( 'Invalid variable dimension: ' +
+                             f'{varx.name}: {varx.maxdim}, ' +
+                             f'{vary.name}: {vary.maxdim}')
 
     # Three Variable Plots
     else:
@@ -147,11 +172,17 @@ def plot(varx   : InVar | OutVar,
                                    invar_space=invar_space,
                                    ax=ax, title=title, plotkwargs=plotkwargs)
 
+        elif varx.maxdim in (0, 1) and vary.maxdim in (0, 1) and varz.maxdim in (0, 1):
+            fig, ax = plot_2p5d_line(varx=varx, vary=vary, varz=varz,
+                                     cases=cases, highlight_cases=highlight_cases,
+                                     invar_space=invar_space,
+                                     ax=ax, title=title, plotkwargs=plotkwargs)
+
         else:
-            raise ValueError( 'Variables have inconsistent dimensions: ' +
-                             f'{varx.name}:{varx.maxdim}, ' +
-                             f'{vary.name}:{vary.maxdim}, ' +
-                             f'{varz.name}:{varz.maxdim}')
+            raise ValueError( 'Invalid variable dimension: ' +
+                             f'{varx.name}: {varx.maxdim}, ' +
+                             f'{vary.name}: {vary.maxdim}, ' +
+                             f'{varz.name}: {varz.maxdim}')
 
     return fig, ax
 
@@ -202,7 +233,7 @@ def plot_hist(var         : InVar | OutVar,
     fig, ax = manage_axis(ax, is3d=False)
     invar_space = manage_invar_space(invar_space=invar_space, nvars=1)
 
-    bins = 'auto'
+    bins : str | np.ndarray = 'auto'
     if 'bins' in plotkwargs:
         bins = plotkwargs['bins']
         del(plotkwargs['bins'])
@@ -380,6 +411,7 @@ def plot_cdf(var       : InVar | OutVar,
 
 def plot_2d_scatter(varx   : InVar | OutVar,
                     vary   : InVar | OutVar,
+                    varz   : InVar | OutVar = None,
                     cases           : None | int | Iterable[int] = None,
                     highlight_cases : None | int | Iterable[int] = empty_list(),
                     rug_plot : bool           = False,
@@ -388,7 +420,7 @@ def plot_2d_scatter(varx   : InVar | OutVar,
                     invar_space : InVarSpace | Iterable[InVarSpace] = InVarSpace.NUMS,
                     ax       : Optional[Axes] = None,
                     title    : str            = '',
-                    plotkwargs  : dict        = dict(),
+                    plotkwargs : dict         = dict(),
                     ) -> tuple[Figure, Axes]:
     """
     Plot a scatter plot of two variables.
@@ -399,6 +431,8 @@ def plot_2d_scatter(varx   : InVar | OutVar,
         The x variable to plot.
     vary : monaco.mc_var.InVar | monaco.mc_var.OutVar
         The y variable to plot.
+    varz : monaco.mc_var.InVar | monaco.mc_var.OutVar, default: None
+        If not None, then sets the values for the underlying contour plot.
     cases : None | int | Iterable[int], default: None
         The cases to plot. If None, then all cases are highlighted.
     highlight_cases : None | int | Iterable[int], default: []
@@ -424,8 +458,12 @@ def plot_2d_scatter(varx   : InVar | OutVar,
         fig is the figure handle for the plot.
         ax is the axes handle for the plot.
     """
+    nvars = 2
+    if varz is not None:
+        nvars = 3
+
     fig, ax = manage_axis(ax, is3d=False)
-    invar_space = manage_invar_space(invar_space=invar_space, nvars=2)
+    invar_space = manage_invar_space(invar_space=invar_space, nvars=nvars)
 
     cases_list = get_cases(varx.ncases, cases)
     highlight_cases_list = get_cases(varx.ncases, highlight_cases)
@@ -433,6 +471,19 @@ def plot_2d_scatter(varx   : InVar | OutVar,
     varx_points = get_plot_points(varx, invar_space[0])
     vary_points = get_plot_points(vary, invar_space[1])
 
+    if varz is not None:
+        ngrid = 200
+        cmap = 'viridis'
+        levels = 21
+
+        xi = np.linspace(min(varx_points), max(varx_points), ngrid)
+        yi = np.linspace(min(vary_points), max(vary_points), ngrid)
+        varz_points = get_plot_points(varz, invar_space[2])
+        zi = griddata((varx_points, vary_points), varz_points,
+                      (xi[None, :], yi[:, None]), method='linear')
+        contour = ax.contourf(xi, yi, zi, levels=levels, cmap=cmap)
+        fig.colorbar(contour, ax=ax, label=varz.name)
+
     if reg_cases:
         plt.scatter(slice_by_index(varx_points, reg_cases),
                     slice_by_index(vary_points, reg_cases),
@@ -610,6 +661,69 @@ def plot_3d_scatter(varx : InVar | OutVar,
 
 
 
+def plot_2p5d_line(varx : InVar | OutVar,
+                   vary : InVar | OutVar,
+                   varz : InVar | OutVar,
+                   cases           : None | int | Iterable[int] = None,
+                   highlight_cases : None | int | Iterable[int] = empty_list(),
+                   invar_space : InVarSpace | Iterable[InVarSpace] = InVarSpace.NUMS,
+                   ax     : Optional[Axes] = None,
+                   title  : str            = '',
+                   plotkwargs : dict       = dict(),
+                   ) -> tuple[Figure, Axes]:
+    """
+    Plot an ensemble of 2.5D lines for one scalar and two nonscalar variables.
+
+    Parameters
+    ----------
+    varx : monaco.mc_var.InVar | monaco.mc_var.OutVar
+        The x variable to plot.
+    vary : monaco.mc_var.InVar | monaco.mc_var.OutVar
+        The y variable to plot.
+    varz : monaco.mc_var.InVar | monaco.mc_var.OutVar
+        The z variable to plot.
+    cases : None | int | Iterable[int], default: None
+        The cases to plot. If None, then all cases are highlighted.
+    highlight_cases : None | int | Iterable[int], default: []
+        The cases to highlight. If [], then no cases are highlighted.
+    invar_space : monaco.InVarSpace | Iterable[InVarSpace], default: 'nums'
+        The space to plot invars in, either 'nums' or 'pcts'. If an iterable,
+        specifies this individually for each of varx, vary, and varz.
+    ax : matplotlib.axes.Axes, default: None
+        The axes handle to plot in. If None, a new figure is created.
+    title : str, default: ''
+        The figure title.
+
+    Returns
+    -------
+    (fig, ax) : (matplotlib.figure.Figure, matplotlib.axes.Axes)
+        fig is the figure handle for the plot.
+        ax is the axes handle for the plot.
+    """
+    var0 = deepcopy(varx)
+    var1 = deepcopy(vary)
+    var2 = deepcopy(varz)
+    npoints = 0
+    for var in (var0, var1, var2):
+        if var.maxdim == 1:
+            npoints = max(npoints, max(len(num) for num in var.nums))
+    for var in (var0, var1, var2):
+        if var.maxdim == 0:
+            for i in range(var.ncases):
+                var.nums[i] = np.array([var.nums[i] for _ in range(npoints)])
+                if isinstance(var, monaco.mc_var.InVar):
+                    var.pcts[i] = [float(var.pcts[i]) for _ in range(npoints)]
+            var.maxdim = 1
+
+    fig, ax = plot_3d_line(varx=var0, vary=var1, varz=var2,
+                           cases=cases, highlight_cases=highlight_cases,
+                           invar_space=invar_space,
+                           ax=ax, title=title, plotkwargs=plotkwargs)
+
+    return fig, ax
+
+
+
 def plot_3d_line(varx : InVar | OutVar,
                  vary : InVar | OutVar,
                  varz : InVar | OutVar,
@@ -854,6 +968,7 @@ def plot_integration_error(outvar       : OutVar,
 
 def plot_sensitivities(outvar        : OutVar,
                        sensitivities : Sensitivities  = Sensitivities.RATIOS,
+                       sort          : bool           = True,
                        ax            : Optional[Axes] = None,
                        title         : str            = '',
                        plotkwargs    : dict           = dict(),
@@ -867,8 +982,10 @@ def plot_sensitivities(outvar        : OutVar,
     ----------
     outvar : monaco.mc_var.OutVar
         The variable representing the integration estimate.
-    sensitivities : monaco.mc_enums.Sensitivities (default: 'ratios')
+    sensitivities : monaco.mc_enums.Sensitivities, default: 'ratios'
         The sensitivities to plot. Either 'ratios' or 'indices'.
+    sort : bool, default: True
+        Whether to show the sensitivity indices sorted from greatest to least.
     ax : matplotlib.axes.Axes, default: None
         The axes handle to plot in. If None, a new figure is created.
     title : str, default: ''
@@ -891,10 +1008,14 @@ def plot_sensitivities(outvar        : OutVar,
         sensitivities_dict = outvar.sensitivity_indices
         ax.set_xlabel(f"Sensitivity Index for '{outvar.name}'")
 
-    sorted_tuples = sorted(sensitivities_dict.items(), key=lambda item: item[1])
-    invarnames = [invarname for invarname, _ in sorted_tuples]
-    sensitivities_vals = [val for _, val in sorted_tuples]
-    y_pos = np.arange(len(invarnames))
+    y_pos = np.arange(len(sensitivities_dict))
+    if sort:
+        sens_tuples = sorted(sensitivities_dict.items(), key=lambda item: item[1])
+    else:
+        sens_tuples = list(sensitivities_dict.items())
+        y_pos = np.flipud(y_pos)
+    invarnames = [invarname for invarname, _ in sens_tuples]
+    sensitivities_vals = [val for _, val in sens_tuples]
 
     ax.barh(y_pos, sensitivities_vals, facecolor='k', alpha=0.5, tick_label=invarnames)
     plt.title(title)
@@ -1162,3 +1283,23 @@ def get_plot_points(var         : InVar | OutVar,
     if isinstance(var, monaco.mc_var.InVar) and invar_space == InVarSpace.PCTS:
         plot_points = var.pcts
     return plot_points
+
+
+
+def get_var_steps(var : InVar | OutVar) -> OutVar:
+    '''
+    For a 1D variable, get an OutVar that has as values the simulation sets.
+
+    Parameters
+    ----------
+    var : InVar | OutVar
+        The target variable.
+
+    Returns
+    -------
+    varsteps : OutVar
+        The variable with simulation steps as values.
+    '''
+    vals = [np.arange(len(num)) for num in var.nums]
+    varsteps = monaco.mc_var.OutVar(name='Simulation Steps', vals=vals)
+    return varsteps
